package lifegame;

public class GameManager {
  /*****************************************************************
   *                          Variables                            *
   *****************************************************************/
  private Grid gameGrid;
  private int  generations;
  
  /*****************************************************************
   *                         Constructors                          *
   *****************************************************************/
  public GameManager(String[] args) throws WrongArgsException, GameManagerCrashedException {
    if (args.length == 3) {
      try {
        int gridSize       = Integer.parseInt(args[0]);
        int initSquareSize = Integer.parseInt(args[1]);
        generations        = Integer.parseInt(args[2]);
        
        if (gridSize < 2)
          throw new WrongArgsException("Invalid data: size of grid should be greater than 1.");
        if (gridSize % 2 != initSquareSize % 2)
          throw new WrongArgsException("Invalid data: size of grid should be of same parity as initial square size.");
        if (gridSize < initSquareSize)
          throw new WrongArgsException("Invalid data: size of grid should be greather than or equal to initial square size.");
        
        gameGrid = new Grid(gridSize, initSquareSize);
      }
      catch (NumberFormatException e) {
        throw new WrongArgsException("There are problems with args types (not integers).");
      }
    }
    else if (args.length == 2) {
      try {
        gameGrid    = new Grid(args[0]);
        generations = Integer.parseInt(args[1]);
      }
      catch (FileErrorException e) {
        throw new GameManagerCrashedException("Error with file primitives: " + e.getMessage());
      }
      catch (ParsingErrorException e) {
        throw new GameManagerCrashedException("Error reading file: " + e.getMessage());
      }
      catch (NumberFormatException e) {
        throw new WrongArgsException("There are problems with args types (generation number not an integer).");
      }
    }
    else
      throw new WrongArgsException("Too few or too many arguments given.");
  }
  
  /*****************************************************************
   *                         Public methods                        *
   *****************************************************************/
  public void runSimulation() throws GameManagerCrashedException {
    System.out.println("Initial situation:");
    gameGrid.display();
    for (int i = 0; i < generations; i++) {
      gameGrid.nextGeneration();
      System.out.println("\nGeneration " + (i + 1) + ":");
      gameGrid.display();
    }
    
    try {
      gameGrid.saveGenerationInFile();
    }
    catch (FileErrorException e) {
      throw new GameManagerCrashedException("Error with file primitives: " + e.getMessage());
    }
  }
  
  /*****************************************************************
   *                        Private methods                        *
   *****************************************************************/
}
