package lifegame;

public class Grid {
  /*****************************************************************
   *                          Variables                            *
   *****************************************************************/
  private Cell[][] Cells;
  private int      size;
  
  /*****************************************************************
   *                         Constructors                          *
   *****************************************************************/
  public Grid(int size, int initSquareSize) {
    this.size = size;
    Cells = new Cell[size][size];
    int beginRand = size / 2 - initSquareSize / 2;
    int endRand   = beginRand + initSquareSize;
    
    for (int i = 0; i < size; i++) {
      for (int j = 0; j < size; j++) {
        if (i >= beginRand && j >= beginRand && i < endRand && j < endRand)
          Cells[i][j] = new Cell(((int)(Math.random() * 100)) % 2 == 0);
        else
          Cells[i][j] = new Cell(false);
      }
    }
    
    initializeCellsNeighborhood();
  }
  
  public Grid(String filename) throws FileErrorException, ParsingErrorException {
    String parsedFile = LifeFile.parseFile(filename);
    size = parsedFile.indexOf('\n');
    autoFillGrid(parsedFile);
    
    initializeCellsNeighborhood();
  }
  
  /*****************************************************************
   *                         Public methods                        *
   *****************************************************************/
  public void saveGenerationInFile() throws FileErrorException {
    LifeFile.writeFile(stringFromState());
  }
  
  public void display() {
    System.out.print(stringFromState());
  }
  
  public void nextGeneration() {
    String newGeneration = "";
    for (int i = 0; i < size; i++) {
      for (int j = 0; j < size; j++) {
        if (Cells[i][j].wantsToDie())
          newGeneration += LifeFile.EmptyCell;
        else if (Cells[i][j].wantsToLive())
          newGeneration += LifeFile.FilledCell;
        else
          newGeneration += (Cells[i][j].isLiving()) ? LifeFile.FilledCell : LifeFile.EmptyCell;
      }
      if (i < size - 1) newGeneration += '\n';
    }
    
    autoFillGrid(newGeneration);
    initializeCellsNeighborhood();
  }
  
  /*****************************************************************
   *                        Private methods                        *
   *****************************************************************/
  private void autoFillGrid(String formattedContent) {
    Cells = new Cell[size][size];
    int counter = 0;
    for (int i = 0; i < size; i++) {
      for (int j = 0; counter < formattedContent.length(); j++) {
          char c = formattedContent.charAt(counter);
          if (c == '\n') break;
          Cells[i][j] = new Cell((c == LifeFile.FilledCell));
          counter++;
      }
      counter++;
    }
  }
  
  private void initializeCellsNeighborhood() {
    Cell[] nearYou;
    
    nearYou = new Cell[3];
    nearYou[0] = Cells[1][0];
    nearYou[1] = Cells[1][1];
    nearYou[2] = Cells[0][1];
    Cells[0][0].createNeighborhood(nearYou);
    
    nearYou = new Cell[3];
    nearYou[0] = Cells[1][size - 1];
    nearYou[1] = Cells[1][size - 2];
    nearYou[2] = Cells[0][size - 2];
    Cells[0][size - 1].createNeighborhood(nearYou);
    
    nearYou = new Cell[3];
    nearYou[0] = Cells[size - 2][0];
    nearYou[1] = Cells[size - 2][1];
    nearYou[2] = Cells[size - 1][1];
    Cells[size - 1][0].createNeighborhood(nearYou);
    
    nearYou = new Cell[3];
    nearYou[0] = Cells[size - 2][size - 1];
    nearYou[1] = Cells[size - 2][size - 2];
    nearYou[2] = Cells[size - 1][size - 2];
    Cells[size - 1][size - 1].createNeighborhood(nearYou);
    
    if (size > 2) {
      for (int i = 1; i < size - 1; i++) {
        nearYou = new Cell[5];
        nearYou[0] = Cells[i - 1][0];
        nearYou[1] = Cells[i - 1][1];
        nearYou[2] = Cells[i][1];
        nearYou[3] = Cells[i + 1][1];
        nearYou[4] = Cells[i + 1][0];
        Cells[i][0].createNeighborhood(nearYou);
        
        nearYou = new Cell[5];
        nearYou[0] = Cells[i - 1][size - 1];
        nearYou[1] = Cells[i - 1][size - 2];
        nearYou[2] = Cells[i][size - 2];
        nearYou[3] = Cells[i + 1][size - 2];
        nearYou[4] = Cells[i + 1][size - 1];
        Cells[i][size - 1].createNeighborhood(nearYou);
        
        nearYou = new Cell[5];
        nearYou[0] = Cells[0][i - 1];
        nearYou[1] = Cells[1][i - 1];
        nearYou[2] = Cells[1][i];
        nearYou[3] = Cells[1][i + 1];
        nearYou[4] = Cells[0][i + 1];
        Cells[0][i].createNeighborhood(nearYou);
        
        nearYou = new Cell[5];
        nearYou[0] = Cells[size - 1][i - 1];
        nearYou[1] = Cells[size - 2][i - 1];
        nearYou[2] = Cells[size - 2][i];
        nearYou[3] = Cells[size - 2][i + 1];
        nearYou[4] = Cells[size - 1][i + 1];
        Cells[size - 1][i].createNeighborhood(nearYou);
      }
      
      for (int i = 1; i < size - 1; i++) {
        for (int j = 1; j < size - 1; j++) {
          nearYou = new Cell[8];
          nearYou[0] = Cells[i - 1][j - 1];
          nearYou[1] = Cells[i - 1][j];
          nearYou[2] = Cells[i - 1][j + 1];
          nearYou[3] = Cells[i][j + 1];
          nearYou[4] = Cells[i + 1][j + 1];
          nearYou[5] = Cells[i + 1][j];
          nearYou[6] = Cells[i + 1][j - 1];
          nearYou[7] = Cells[i][j - 1];
          Cells[i][j].createNeighborhood(nearYou);
        }
      }
    }
  }
  
  private String stringFromState() {
    String toReturn = "";
    for (int i = 0; i < size; i++) {
      for (int j = 0; j < size; j++) {
          if (Cells[i][j].isLiving())
            toReturn += LifeFile.FilledCell;
          else
            toReturn += LifeFile.EmptyCell;
      }
      if (i < size - 1) toReturn += '\n';
    }
    
    return toReturn;
  }
}
