package lifegame;
import java.io.*;

public class LifeFile {
  /*****************************************************************
   *                          Variables                            *
   *****************************************************************/
  public final static char FilledCell = '*';
  public final static char EmptyCell = '.';
  
  private static FileInputStream   fis = null;
  private static InputStreamReader isr = null;
  private static BufferedReader    br  = null;
  private static FileWriter        fw  = null;
  private static BufferedWriter    bw  = null;
  
  /*****************************************************************
   *                         Constructors                          *
   *****************************************************************/
  private LifeFile() {} // This class cannot be instanced: class methods only
  
  /*****************************************************************
   *                         Public methods                        *
   *****************************************************************/
  public static String parseFile(String filename) throws FileErrorException, ParsingErrorException {
    // -- Opening file --
    try {
      fis = new FileInputStream(filename);
      isr = new InputStreamReader(fis);
      br  = new BufferedReader(isr);
    }
    catch (IOException e) {
      closeStreams();
      throw new FileErrorException("Opening of file '" + filename + "' failed.");
    }
    
    // -- Parsing file --
    String parsedFile = "";
    int  charCode;
    char readChar;
    
    try {
      // Read first line and determine awaited size of the grid
      while ((charCode = br.read()) != '\n') {
        readChar = (char)charCode;
        if (readChar == FilledCell || readChar == EmptyCell)
          parsedFile += readChar;
        else if (charCode == -1)
          throw new ParsingErrorException("Invalid file format - grid size should be greater than 1.");
        else
          throw new ParsingErrorException("Invalid file format - wrong character read: '" + readChar + "'.");
      }
      
      int size = parsedFile.length();
      parsedFile += '\n';
      
      // While EOF not reached, read characters
      int countedCols = 0;
      int countedRows = 1;
      while ((charCode = br.read()) != -1) {
        readChar = (char)charCode;
        if (readChar == FilledCell || readChar == EmptyCell) {
          if (++countedCols > size) 
            throw new ParsingErrorException("Invalid file format - lines have not the same length).");
          parsedFile += readChar;
        }
        else if (readChar == '\n') {
          if (++countedRows >= size)
            throw new ParsingErrorException("Invalid file format - too much lines.");
          parsedFile += readChar;
          countedCols = 0;
        }
        else
          throw new ParsingErrorException("Invalid file format - wrong character read: '" + readChar + "'.");
      }
      
      // If not enough lines, EOF reached too soon : countedRows < size.
      if (countedRows + 1 < size)
        throw new ParsingErrorException("Invalid file format - too few lines.");
    }
    catch (IOException e) {
      throw new FileErrorException("Reading file '" + filename + "' failed.");
    }
    finally {
      closeStreams();
    }
    
    return parsedFile;
  }
  
  public static void writeFile(String formattedContent) throws FileErrorException {
    try {
      // Create file 
      fw = new FileWriter("fin.txt");
      bw = new BufferedWriter(fw);
    }
    catch (IOException e) {
      closeStreams();
      throw new FileErrorException("Opening and creation of file 'fin.txt' failed.");
    }
    
    try {
      // Write file
      bw.write(formattedContent);
    }
    catch (IOException e) {
      throw new FileErrorException("Writing file 'fin.txt' failed.");
    }
    finally {
      closeStreams();
    }
  }
  
  /*****************************************************************
   *                        Private methods                        *
   *****************************************************************/
  private static void closeStreams() {
    try {
      if (br  != null) br.close();
      if (isr != null) isr.close();
      if (fis != null) fis.close();
      if (bw  != null) bw.close();
      if (fw  != null) fw.close();
    }
    catch (IOException e) {}
    finally {
      br  = null;
      isr = null;
      fis = null;
      bw  = null;
      fw  = null;
    }
  }
}
